"""Modeling components"""

from ..common import method_decorator
from . import fragments
from .ast import (
    Cross,
    Domain,
    Expression,
    ExpressionLike,
    IterableSpace,
    Predicate,
    Projection,
    Quantifiable,
    Quantification,
    Quantifier,
    Space,
    cross,
    domain,
    literal,
    size,
    switch,
    total,
)
from .definitions import (
    Constraint,
    Dimension,
    Image,
    Objective,
    Parameter,
    Tensor,
    TensorLike,
    Variable,
    alias,
    constraint,
    interval,
    objective,
)
from .model import Definition, Model, ModelFragment, Statement, relabel
from .quantified import Quantified

__all__ = [
    "Model",
    "ModelFragment",
    "Statement",
    "relabel",
    # Definitions
    "Constraint",
    "Definition",
    "Dimension",
    "Image",
    "Objective",
    "Parameter",
    "Tensor",
    "TensorLike",
    "Variable",
    "alias",
    "constraint",
    "objective",
    # Expressions and predicates
    "Expression",
    "ExpressionLike",
    "Predicate",
    "interval",
    "literal",
    "size",
    "switch",
    "total",
    # Quantification
    "Cross",
    "IterableSpace",
    "Domain",
    "Projection",
    "Quantifiable",
    "Quantification",
    "Quantified",
    "Quantifier",
    "Space",
    "cross",
    "domain",
    # Fragments
    "fragments",
    # Utilities
    "method_decorator",
]



import logging
import os
import threading
import types
from typing import Optional
import warnings

from ..modeling import Model


_logger = logging.getLogger(__name__)


def load_notebook_models(
    path: str, root: Optional[str] = None, allow_empty=False
) -> types.SimpleNamespace:
    """Loads all models from a notebook

    Args:
        path: Path to the notebook, relative to `root` if present otherwise CWD
        root: Root path. If set to a file, its parent directory will be used
            (convenient for use with `__file__`).
        allow_empty: Do not throw an error if no models were found
    """
    if root:
        root = os.path.realpath(root)
        if os.path.isfile(root):
            root = os.path.dirname(root)
        path = os.path.join(root, path)
    ns = types.SimpleNamespace()

    # We run the import logic in a separate, fresh, thread since `importnb`
    # expects an inactive event loop if the notebook includes async statements.
    t = _ImportThread(target=_populate_notebook_namespace, args=(path, ns))
    t.start()
    t.join()

    if not ns.__dict__ and not allow_empty:
        raise Exception(f"No models found in {path}")
    return ns


class _ImportThread(threading.Thread):
    """Thread which rethrows any import exception"""

    _exception = None

    def run(self):
        try:
            super().run()
        except Exception as e:
            self._exception = e

    def join(self):
        super().join()
        if self._exception:
            raise Exception("Notebook import failed") from self._exception


def _populate_notebook_namespace(path: str, ns: types.SimpleNamespace) -> None:
    import asyncio

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)

    with warnings.catch_warnings():
        warnings.filterwarnings(
            action="ignore",
            category=DeprecationWarning,
            module=r".*(importnb|IPython)",
        )
        import importnb  # type: ignore

    class _Notebook(importnb.Notebook):
        def code(self, raw):
            # We skip magic cells (this is done manually since the default
            # `no_magic` option only skips cells starting with 2 %).
            if raw.startswith("%"):
                return "# " + raw
            # We use a custom loader to transform all top-level awaited
            # expressions into statements, otherwise their value will show up
            # in importing notebooks. Note that this isn't fool-proof since we
            # rely on the `await` keyword being first on the line.
            lines = [
                f"_ = {s}" if s.startswith("await ") else s
                for s in raw.split("\n")
            ]
            return super().code("\n".join(lines))

    nb = _Notebook.load_file(path)

    count = 0
    for attr in dir(nb):
        value = getattr(nb, attr)
        if isinstance(value, Model):
            count += 1
            setattr(ns, attr, value)
    _logger.debug("Loaded %s model(s) from %s.", count, path)



import contextlib
import dataclasses
import json
import logging
from typing import (
    Any,
    AsyncContextManager,
    AsyncIterator,
    Type,
    Mapping,
    Optional,
    TypeVar,
)
import urllib.parse

from ..common import __version__


_logger = logging.getLogger(__name__)


TRACE_HEADER = "opvious-trace"


AUTHORIZATION_HEADER = "authorization"


CONTENT_TYPE_HEADER = "content-type"


Headers = dict[str, str]


@dataclasses.dataclass(init=False)
class ExecutorError(Exception):
    """Local representation of an error during an executor's request"""

    status: int
    trace: Optional[str]
    reason: Optional[str]

    def __init__(
        self,
        status: int,
        trace: Optional[str] = None,
        reason: Optional[Any] = None,
    ) -> None:
        message = f"Request failed with status {status}"
        if trace:
            message += f" ({trace})"
        if reason:
            message += f": {reason}"
        super().__init__(message)
        self.status = status
        self.trace = trace
        self.reason = reason


def unexpected_response_error(
    message: str, trace: Optional[str] = None
) -> Exception:
    return Exception(
        "Unexpected response"
        + (f" ({trace})" if trace else "")
        + (f": {message}" if message else "")
    )


def unsupported_content_type_error(
    text: str, content_type: Optional[str], trace: Optional[str] = None
) -> Exception:
    return unexpected_response_error(
        message=(
            f"unsupported content-type ({content_type or '<none>'}): {text}"
        ),
        trace=trace,
    )


@dataclasses.dataclass
class ExecutorResult:
    """Request execution result"""

    status: int
    """Response HTTP status code"""

    trace: Optional[str]
    """Request trace ID"""

    def __post_init__(self):
        _logger.debug(
            "Got executor result. [status=%s, trace=%s]",
            self.status,
            self.trace,
        )

    @property
    def accept(self) -> str:
        raise NotImplementedError()

    @property
    def content_type(self) -> str:
        raise NotImplementedError()

    def _assert_status(self, status: int, text: Optional[str] = None) -> None:
        if self.status == status:
            return
        raise ExecutorError(status=self.status, trace=self.trace, reason=text)

    @classmethod
    def is_eligible(cls, ctype: Optional[str]) -> bool:
        return bool(ctype and ctype.split(";")[0] == cls.content_type)


@dataclasses.dataclass
class PlainTextExecutorResult(ExecutorResult):
    """Plain text execution result"""

    content_type = "text/plain"
    reader: Any = dataclasses.field(repr=False)

    async def text(self, assert_status: Optional[int] = None) -> str:
        lines = []
        async for line in self.lines(assert_status=assert_status):
            lines.append(line)
        return "".join(lines)

    async def lines(
        self, assert_status: Optional[int] = 200
    ) -> AsyncIterator[str]:
        if assert_status:
            self._assert_status(assert_status)

        # Non-streaming
        if isinstance(self.reader, str):
            for line in self.reader.splitlines(keepends=True):
                yield line
            return

        # Streaming
        splitter = _LineSplitter()
        if hasattr(self.reader, "__aiter__"):
            async for chunk in self.reader:
                for line in splitter.push(chunk):
                    yield line
        elif hasattr(self.reader, "__iter__"):
            for chunk in self.reader:
                for line in splitter.push(chunk):
                    yield line
        else:
            raise Exception(f"Non-iterable reader: {self.reader}")
        yield splitter.flush()


class _LineSplitter:
    def __init__(self):
        self._buffer = ""

    def push(self, chunk: bytes) -> list[str]:
        lines = chunk.decode("utf8").splitlines(keepends=True)
        if self._buffer:
            lines[0] = self._buffer + lines[0]
        if lines[-1].endswith("\n"):
            self._buffer = ""
        else:
            self._buffer = lines.pop()
        return lines

    def flush(self) -> str:
        buf = self._buffer
        self._buffer = ""
        return buf


@dataclasses.dataclass
class JsonExecutorResult(ExecutorResult):
    """Unary JSON execution result"""

    content_type = "application/json"
    text: str = dataclasses.field(repr=False)

    def json_data(self, status: int = 200) -> Any:
        self._assert_status(status, self.text)
        return json.loads(self.text)


@dataclasses.dataclass
class JsonSeqExecutorResult(ExecutorResult):
    """Streaming JSON execution result"""

    content_type = "application/json-seq"
    reader: Any = dataclasses.field(repr=False)

    async def json_seq_data(self) -> AsyncIterator[Any]:
        self._assert_status(200)
        if hasattr(self.reader, "__aiter__"):
            async for line in self.reader:
                yield _json_seq_item(line)
        elif hasattr(self.reader, "__iter__"):
            # For synchronous executors
            for line in self.reader:
                yield _json_seq_item(line)
        else:
            raise Exception(f"Non-iterable reader: {self.reader}")


RECORD_SEPARATOR = b"\x1e"


def _json_seq_item(line: bytes) -> Any:
    # TODO: Robust error checking.
    data = line[1:] if line.startswith(RECORD_SEPARATOR) else line
    return json.loads(data)


ExpectedExecutorResult = TypeVar(
    "ExpectedExecutorResult", bound=ExecutorResult
)


class Executor:
    """Generic HTTP request executor"""

    def __init__(
        self,
        variant: str,
        endpoint: str,
        authorization: Optional[str] = None,
        supports_streaming=False,
    ):
        self._variant = variant
        self._endpoint = endpoint
        self._root_headers = _default_headers(variant)
        if authorization:
            self._root_headers[AUTHORIZATION_HEADER] = authorization
        self.supports_streaming = supports_streaming
        _logger.debug("Instantiated %s executor. [url=%s]", variant, endpoint)

    @property
    def endpoint(self) -> str:
        """The executor's root endpoint, used for all relative URLs"""
        return self._endpoint

    @property
    def authenticated(self):
        """Whether requests use an API authorization header"""
        return AUTHORIZATION_HEADER in self._root_headers

    def _send(
        self, url: str, method: str, headers: Headers, body: Optional[bytes]
    ) -> AsyncContextManager[ExecutorResult]:
        raise NotImplementedError()

    @contextlib.asynccontextmanager
    async def execute(
        self,
        result_type: Type[ExpectedExecutorResult],
        url: str,
        method: str = "GET",
        headers: Optional[Headers] = None,
        json_data: Optional[Any] = None,
    ) -> AsyncIterator[ExpectedExecutorResult]:
        """Send a request"""
        full_url = urllib.parse.urljoin(self.endpoint, url)
        if full_url.startswith(self.endpoint):
            all_headers = self._root_headers.copy()
        else:
            all_headers = {}
        if headers:
            all_headers.update(headers)

        if result_type == JsonExecutorResult:
            accept = "application/json;q=1, text/plain;q=0.1"
        elif result_type == JsonSeqExecutorResult:
            accept = "application/json-seq;q=1, text/plain;q=0.1"
        elif result_type == PlainTextExecutorResult:
            accept = "text/plain"
        else:
            raise Exception(f"Unsupported result type: {result_type}")
        all_headers["accept"] = accept

        if json_data:
            all_headers["content-type"] = "application/json"
            body = json.dumps(json_data).encode("utf8")
        else:
            body = None

        _logger.debug(
            "Sending %s executor request... [size=%s, url=%s]",
            self._variant,
            len(body) if body else 0,
            full_url,
        )
        async with self._send(
            url=full_url,
            method=method,
            headers=all_headers,
            body=body,
        ) as result:
            if not isinstance(result, result_type):
                if isinstance(result, PlainTextExecutorResult):
                    reason = await result.text()
                elif isinstance(result, JsonExecutorResult):
                    reason = result.text
                else:
                    reason = None
                raise ExecutorError(
                    status=result.status, trace=result.trace, reason=reason
                )
            yield result

    async def execute_graphql_query(
        self,
        query: str,
        variables: Optional[Mapping[str, Any]] = None,
    ) -> Any:
        """Send a GraphQL API request"""
        async with self.execute(
            result_type=JsonExecutorResult,
            url="/graphql",
            method="POST",
            json_data={"query": query, "variables": variables or {}},
        ) as result:
            data = result.json_data()
        if data.get("errors"):
            raise ExecutorError(
                status=result.status,
                trace=result.trace,
                reason=json.dumps(data),
            )
        return data["data"]


def _default_headers(client: str) -> Headers:
    return {
        "opvious-client": f"Python SDK v{__version__} ({client})",
    }



from __future__ import annotations

import dataclasses
from datetime import datetime
from typing import Any, Optional

from .outlines import ProblemOutline
from .tensors import Value


@dataclasses.dataclass(frozen=True)
class QueuedSolve:
    """Queued optimization attempt

    Solves are queued via :meth:`.Client.queue_solve`, existing queued solves
    can be retrieved from their UUID via :meth:`.Client.fetch_solve`.
    """

    uuid: str
    """The solve's unique identifier"""

    started_at: datetime
    """The time the solve was created"""

    outline: ProblemOutline = dataclasses.field(repr=False)
    """The specification outline corresponding to this solve"""


def queued_solve_from_graphql(
    data: Any, outline: ProblemOutline
) -> QueuedSolve:
    return QueuedSolve(
        uuid=data["uuid"],
        started_at=datetime.fromisoformat(data["startedAt"]),
        outline=outline,
    )


@dataclasses.dataclass(frozen=True)
class SolveNotification:
    """Solve progress update notification"""

    dequeued: bool
    """Whether the solve has already been dequeued"""

    relative_gap: Optional[Value]
    """The latest relative gap"""

    lp_iteration_count: Optional[int]
    """The latest LP iteration count"""

    cut_count: Optional[int]
    """The latest cut count"""


def solve_notification_from_graphql(
    dequeued: bool, data: Any = None
) -> SolveNotification:
    return SolveNotification(
        dequeued=dequeued,
        relative_gap=data["relativeGap"] if data else None,
        lp_iteration_count=data["lpIterationCount"] if data else None,
        cut_count=data["cutCount"] if data else None,
    )



import aiohttp
import brotli  # type: ignore
import contextlib
import logging
from typing import AsyncIterator, Optional

from .common import (
    CONTENT_TYPE_HEADER,
    Executor,
    ExecutorResult,
    JsonExecutorResult,
    JsonSeqExecutorResult,
    PlainTextExecutorResult,
    Headers,
    TRACE_HEADER,
    unexpected_response_error,
    unsupported_content_type_error,
)


_logger = logging.getLogger(__name__)


_BROTLI_QUALITY = 4


_COMPRESSION_THRESHOLD = 2**16


_READ_BUFFER_SIZE = 2**26  # 512MiB


_REQUEST_TIMEOUT_SECONDS = 900  # 15 minutes


class AiohttpExecutor(Executor):
    """`aiohttp`-powered executor"""

    def __init__(self, endpoint: str, authorization: Optional[str] = None):
        super().__init__(
            variant="aiohttp",
            endpoint=endpoint,
            authorization=authorization,
            supports_streaming=True,
        )

    @contextlib.asynccontextmanager
    async def _send(
        self, url: str, method: str, headers: Headers, body: Optional[bytes]
    ) -> AsyncIterator[ExecutorResult]:
        if body and len(body) > _COMPRESSION_THRESHOLD:
            headers["content-encoding"] = "br"
            compressed_body = brotli.compress(
                body,
                mode=brotli.MODE_TEXT,
                quality=_BROTLI_QUALITY,
            )
            _logger.debug(
                "Compressed request body. [size=%s]", len(compressed_body)
            )
            body = compressed_body
        try:
            async with aiohttp.ClientSession(
                headers=headers,
                read_bufsize=_READ_BUFFER_SIZE,
                timeout=aiohttp.ClientTimeout(total=_REQUEST_TIMEOUT_SECONDS),
            ) as session:
                async with session.request(
                    url=url, method=method, data=body
                ) as res:
                    status = res.status
                    trace = res.headers.get(TRACE_HEADER)
                    ctype = res.headers.get(CONTENT_TYPE_HEADER)
                    if JsonExecutorResult.is_eligible(ctype):
                        text = await res.text()
                        yield JsonExecutorResult(
                            status=status,
                            trace=trace,
                            text=text,
                        )
                    elif JsonSeqExecutorResult.is_eligible(ctype):
                        yield JsonSeqExecutorResult(
                            status=status,
                            trace=trace,
                            reader=res.content,
                        )
                    elif PlainTextExecutorResult.is_eligible(ctype):
                        yield PlainTextExecutorResult(
                            status=status,
                            trace=trace,
                            reader=res.content,
                        )
                    else:
                        text = await res.text()
                        raise unsupported_content_type_error(
                            text=text,
                            content_type=ctype,
                            trace=trace,
                        )
        except aiohttp.ClientResponseError as err:
            trace = None
            if isinstance(err.headers, list):
                trace = next(
                    (v for (k, v) in err.headers if k == TRACE_HEADER), None
                )
            raise unexpected_response_error(message=err.message, trace=trace)



from __future__ import annotations

import dataclasses
from typing import Literal, Mapping, Optional

from ..common import decode_extended_float, Json
from .tensors import is_value, Value


#: Model name
Label = str
"""Model component name"""


@dataclasses.dataclass(frozen=True)
class SourceBinding:
    """Parameter key item binding"""

    dimension_label: Optional[Label]
    """The label of the dimension if the key item corresponds to one"""

    qualifier: Optional[Label]
    """The binding's qualifier, if any"""


def _source_binding_from_json(data: Json) -> SourceBinding:
    return SourceBinding(
        dimension_label=data.get("dimensionLabel"),
        qualifier=data.get("qualifier"),
    )


ObjectiveSense = Literal[
    "MAXIMIZE",
    "MINIMIZE",
]
"""Objective direction"""


@dataclasses.dataclass(frozen=True)
class ObjectiveOutline:
    """Objective metadata"""

    label: Label
    """The objective's unique label"""

    sense: ObjectiveSense
    """Whether this is a maximization (or minimization)"""

    is_quadratic: bool
    """Whether the objective has any quadratic coefficients"""


def _objective_from_json(data: Json) -> ObjectiveOutline:
    return ObjectiveOutline(
        label=data["label"],
        sense="MAXIMIZE" if data["isMaximization"] else "MINIMIZE",
        is_quadratic=data["isQuadratic"],
    )


@dataclasses.dataclass(frozen=True)
class DimensionOutline:
    """Dimension metadata"""

    label: Label
    """The dimension's unique label"""

    is_numeric: bool
    """Whether the dimension contains numeric items"""


def _dimension_from_json(data: Json) -> DimensionOutline:
    return DimensionOutline(label=data["label"], is_numeric=data["isNumeric"])


@dataclasses.dataclass(frozen=True)
class TensorOutline:
    """Parameter or variable metadata"""

    label: Label
    """The tensor's unique label"""

    lower_bound: Optional[Value]
    """The tensor's lower bound if it is statically known"""

    upper_bound: Optional[Value]
    """The tensor's upper bound if it is statically known"""

    is_integral: bool
    """Whether the tensor contains only integer values"""

    bindings: list[SourceBinding]
    """Key bindings"""

    derivation_kind: Optional[str]
    """Derived tensor kind if applicable"""

    @property
    def is_indicator(self) -> bool:
        """Whether the tensor is guaranteed to contain only 0s and 1s"""
        return (
            self.is_integral
            and self.lower_bound == 0
            and self.upper_bound == 1
        )


def _tensor_from_json(data: Json) -> TensorOutline:
    img = data["image"]
    lb = decode_extended_float(img["lowerBound"])
    ub = decode_extended_float(img["upperBound"])
    dv = data.get("derivation")
    return TensorOutline(
        label=data["label"],
        lower_bound=lb if is_value(lb) else None,
        upper_bound=ub if is_value(ub) else None,
        is_integral=img["isIntegral"],
        bindings=[_source_binding_from_json(b) for b in data["bindings"]],
        derivation_kind=dv["kind"] if dv else None,
    )


@dataclasses.dataclass(frozen=True)
class ConstraintOutline:
    """Constraint metadata"""

    label: Label
    """The constraint's unique label"""

    bindings: list[SourceBinding]
    """Quantifier key bindings"""


def _constraint_from_json(data: Json) -> ConstraintOutline:
    return ConstraintOutline(
        label=data["label"],
        bindings=[_source_binding_from_json(b) for b in data["bindings"]],
    )


@dataclasses.dataclass(frozen=True)
class ProblemOutline:
    """Model metadata"""

    objectives: Mapping[Label, ObjectiveOutline]
    """Objective metadata, if applicable"""

    dimensions: Mapping[Label, DimensionOutline]
    """Dimension metadata, keyed by dimension label"""

    parameters: Mapping[Label, TensorOutline]
    """Parameter metadata, keyed by parameter label"""

    variables: Mapping[Label, TensorOutline]
    """Variable metadata, keyed by variable label"""

    constraints: Mapping[Label, ConstraintOutline]
    """Constraint metadata, keyed by constraint label"""


def outline_from_json(data: Json) -> ProblemOutline:
    return ProblemOutline(
        objectives=_map_outlines(_objective_from_json, data["objectives"]),
        dimensions=_map_outlines(_dimension_from_json, data["dimensions"]),
        parameters=_map_outlines(_tensor_from_json, data["parameters"]),
        variables=_map_outlines(_tensor_from_json, data["variables"]),
        constraints=_map_outlines(_constraint_from_json, data["constraints"]),
    )


def _map_outlines(from_json, data):
    return {o["label"]: from_json(o) for o in data}



from typing import Union

from .local import (
    LocalSpecification,
    LocalSpecificationIssue,
    LocalSpecificationSource,
    local_specification_issue_from_json,
)
from .external import FormulationSpecification, RemoteSpecification
from .notebook import load_notebook_models


Specification = Union[
    LocalSpecification,
    RemoteSpecification,
    FormulationSpecification,
]


__all__ = [
    "FormulationSpecification",
    "LocalSpecification",
    "LocalSpecificationIssue",
    "LocalSpecificationSource",
    "RemoteSpecification",
    "Specification",
    "local_specification_issue_from_json",
    "load_notebook_models",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['opvious',
 'opvious.client',
 'opvious.data',
 'opvious.executors',
 'opvious.modeling',
 'opvious.specifications']

package_data = \
{'': ['*']}

install_requires = \
['backoff>=2.2,<3.0',
 'humanize>=4.4.0,<5.0.0',
 'importnb>=2023.1.7,<2024.0.0',
 'numpy>=1.21',
 'pandas>=1.4,<2.0']

extras_require = \
{'aio': ['aiohttp>=3.8,<4.0', 'Brotli>=1.0.9,<2.0.0'],
 'cli': ['docopt>=0.6.2,<0.7.0']}

setup_kwargs = {
    'name': 'opvious',
    'version': '0.18.3rc1',
    'description': 'Opvious Python SDK',
    'long_description': '# Opvious Python SDK  [![CI](https://github.com/opvious/sdk.py/actions/workflows/ci.yml/badge.svg)](https://github.com/opvious/sdk.py/actions/workflows/ci.yml) [![Pypi badge](https://badge.fury.io/py/opvious.svg)](https://pypi.python.org/pypi/opvious/)\n\nAn SDK for solving linear, mixed-integer, and quadratic optimization models via\nthe [Opvious](https://www.opvious.io) API.\n\n## Highlights\n\n### Declarative modeling API\n\n+ Extensive static validations\n+ Exportable to LaTeX\n+ Extensible support for high-level patterns (activation variables, masks, ...)\n\n```python\nimport opvious.modeling as om\n\nclass BinPacking(om.Model):\n  items = om.Dimension()  # All items to bin\n  weight = om.Parameter.non_negative(items)  # Weight per item\n  bins = om.interval(1, om.size(items), name="B")  # Available bins\n  max_weight = om.Parameter.non_negative()  # Maximum weight for each bin\n  assigned = om.Variable.indicator(bins, items)  # Bin to item assignment\n  used = om.fragments.ActivationVariable(assigned, projection=1)  # 1 if a bin is used\n\n  @om.constraint\n  def each_item_is_assigned_once(self):\n    for i in self.items:\n      yield om.total(self.assigned(b, i) for b in self.bins) == 1\n\n  @om.constraint\n  def bin_weights_are_below_max(self):\n    for b in self.bins:\n      bin_weight = om.total(self.weight(i) * self.assigned(b, i) for i in self.items)\n      yield bin_weight <= self.max_weight()\n\n  @om.objective\n  def minimize_bins_used(self):\n    return om.total(self.used(b) for b in self.bins)\n```\n\nAuto-generated specification:\n\n<p align="center">\n  <img alt="Bin package LaTeX specification" src="resources/images/bin-packing-specification.png" width="600px">\n</p>\n\n\n### Transparent remote solves\n\n+ No local solver installation required\n+ Real-time progress notifications\n+ Seamless data import/export via native support for `pandas`\n+ Flexible multi-objective support: weighted sums, epsilon constraints, ...\n+ Built-in debugging capabilities: relaxations, fully annotated LP formatting,\n  ...\n\n```python\nimport opvious\n\nclient = opvious.Client.default()\n\nsolution = await client.solve(\n  opvious.Problem(\n    specification=BinPacking().specification(),\n    parameters={\n      "weight": {"a": 10.5, "b": 22, "c": 48},\n      "binMaxWeight": 50,\n    },\n  ),\n)\nassignments = solution.outputs.variable("assigned") # Optimal values\n```\n\nTake a look at https://opvious.readthedocs.io for the full documentation or\n[these notebooks][notebooks] to see the SDK in action.\n\n[notebooks]: https://github.com/opvious/examples/tree/main/notebooks\n',
    'author': 'Opvious Engineering',
    'author_email': 'oss@opvious.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/opvious/sdk.py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<3.12',
}


setup(**setup_kwargs)
